c========================================================================
      PROGRAM FLUX_MODEL_OUTER
c========================================================================
c
c    AUTHOR: Michael H. Denton
c    Copyright 2019 Michael H. Denton
c    Version 1.0 : 13/07/2019
c
c    GNU-GPL v3 (see below and file gpl.txt, which accompanies this code)
c
c    A full summary of the model can be found in the publication that 
c    accompanies this code:
c
c    "Extension of an empirical electron flux model from 6 to 20 Earth 
c    radii using CLUSTER/RAPID observations."
c    M. H. Denton, M. G. G. T. Taylor, J. V. Rodriguez, M. G. Henderson.
c    Space Weather, 17, 778–792, doi:10.1029/2018SW002121, 2019.
c
c    The help and suggestions of co-authors is gratefully acknowledged!!!
c
c    If you find the code useful, or have suggestions for changes, then 
c    please let me know.  Mick Denton - mdenton@spacescience.org
c
c=============================PROGRAM OVERVIEW===========================
c
c    The code returns electron flux values between 6-20 Earth radii
c    as a function of energy, magnetic local-time, and Kp index.  
c    The underlying data for the model is from the RAPID instrument
c    on board the Cluster spacecraft.
c
c    Model bounds: ~45 keV to ~325 keV, any MLT, any Kp, any radii (6-20)
c
c    MODEL BINNING GRID: 6xenergy, 6xKp, 24xMLT, 20xradial distances.
c
c    INPUTS:
c    The user may enter the following:
c
c      choose_en:  Energy in eV,
c                  e.g. 25500=25.5 keV.
c
c      choose_kp:  Any Kp value 
c
c      choose_lt:  Magnetic Local Time
c                  (e.g. 13.80 MLT)
c
c      choose_rd:  Radius in Earth Radii 
c                  (e.g. 6.6 = Geosynchronous orbit)
c
c    OUTPUTS: 
c    The model returns the following:
c
c  Mean electron flux: 
c  Standard deviation on the mean electron flux
c   5th percentile electron flux (five)
c  25th percentile electron flux (tfive)
c  50th percentile electron flux (med)
c  75th percentile electron flux (sfive)
c  95th percentile electron flux (nfive)
c    
c  Fluxes are given in units of (cm^-2 s^-1 sr^-1 eV^-1)
c
c    DESCRIPTION OF SUBROUTINES
c  
c    flux_model_outer.f: 
c          The MAIN subroutine.
c    bilinear_outer.f: 
c          Function which performs bilinear interpolation.
c    calc_outer.f:
c          Subroutine which takes the input and returns flux values.
c    data_read_ouuter.f:
c          Subroutine to read in the underlying CLUSTER data files.
c
c    GNU GENERAL PUBLIC LICENSE
c    This program is free software: you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation, either version 3 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details (file gpl.txt).
c
c    You should have received a copy of the GNU General Public License
c    along with this program.  If not, see <http://www.gnu.org/licenses/>
c
c    Please feel free to modify the code to suit your own needs in line 
c    with this license.
c
c========================================================================

      IMPLICIT NONE

c     DEFINE INTEGER VARIABLES
      INTEGER(KIND=8) RADS, KPS, ENS, LTS, VALS
      INTEGER(KIND=8) I, J, K, L
      INTEGER(KIND=8) cols, output_cols
      INTEGER(KIND=8) ARR_SIZE, choose_energy_index

c     DEFINE PARAMETERS
      PARAMETER(RADS=20, KPS=6, ENS=6, LTS=24, VALS=7)
      PARAMETER(cols=4, output_cols=20)
      PARAMETER(ARR_SIZE=10)

c     DEFINE REAL VARIABLES
      REAL(KIND=8) choose_rd, choose_kp, choose_en, choose_lt
      INTEGER(KIND=8) this_kp

c     DEFINE CHARACTER ARRAYS
      CHARACTER head_arr1(1)*60
      CHARACTER head_arr2(1)*20
      CHARACTER head_arr3(1)*20

c     DEFINE INTEGER ARRAYS
      INTEGER(KIND=8) kp_array(KPS)

c     DEFINE CHARACTER ARRAYS
      CHARACTER str_kp(KPS)*2
      CHARACTER str_arr(VALS)*10
      CHARACTER head_arr(1)*16
      CHARACTER*100 model_input_file, model_output_file, header

c     DEFINE REAL ARRAYS
      REAL(KIND=8) mean_e_all(RADS,LTS,KPS,ENS)
      REAL(KIND=8) std_e_all(RADS,LTS,KPS,ENS)
      REAL(KIND=8) five_e_all(RADS,LTS,KPS,ENS)
      REAL(KIND=8) tfive_e_all(RADS,LTS,KPS,ENS)
      REAL(KIND=8) med_e_all(RADS,LTS,KPS,ENS)
      REAL(KIND=8) sfive_e_all(RADS,LTS,KPS,ENS)
      REAL(KIND=8) nfive_e_all(RADS,LTS,KPS,ENS)

      REAL(KIND=8) final_e_flux_all(VALS)
      REAL(KIND=8) input_arr(cols,ARR_SIZE)
c      REAL(KIND=8) choose_year, choose_doy, choose_ut
      REAL(KIND=8) data_flux, full_kp
      REAL(KIND=8) energy_array(ENS)

      DATA str_kp/ 
     >             '0', '1o', '2o', '3o', '4o', '5o' /

      DATA kp_array/ 
     >               1, 2, 3, 4, 5, 6 /

      DATA energy_array/
c    CLUSTER
     > 45000.D0, 59000.D0, 81000.D0, 110000.D0, 185000.D0, 325000.D0/


      DATA str_arr/
     >'      MEAN','   STD-DEV','   5TH PER','  25th PER','    MEDIAN',
     >'  75th PER','  95th PER'/

      DATA head_arr1/
     >'R_E   KP   EN   LT'/
      DATA head_arr2/
     >'ELECTRONS'/

c     ===================================================================
c     --------------END DEFINITIONS-BEGIN MODEL--------------------------
c     ===================================================================

c     PRINT HEADER TO SCREEN
      WRITE(6,*) ''
      WRITE(6,*) '====================================================='
      WRITE(6,*) '   AUTHOR:   Michael H. Denton'
      WRITE(6,*) '   CODE:     FLUX_MODEL_OUTER'
      WRITE(6,*) '   VERSION:  1.0(13/06/2019)'
      WRITE(6,*) ''
      WRITE(6,*) '      flux_model_outer.f'
      WRITE(6,*) '        bilinear_outer.f'
      WRITE(6,*) '            calc_outer.f'
      WRITE(6,*) '       data_read_outer.f'
      WRITE(6,*) ''
      WRITE(6,*) '  This is free software under the terms of the GNU '
      WRITE(6,*) '  General Public License v3.  You are welcome to '
      WRITE(6,*) '  redistribute it under certain conditions. You'
      WRITE(6,*) '  should have received a copy of the GNU General'
      WRITE(6,*) '  Public License along with this program. If not, see'
      WRITE(6,*) '  http://gnu.org/licenses'
      WRITE(6,*) ''
      WRITE(6,*) '  Feel free to modify in accordance with the license!'
      WRITE(6,*) ''
      WRITE(6,*) '  This program comes with ABSOLUTELY NO WARRANTY!!!'
      WRITE(6,*) '====================================================='
      WRITE(6,*) ''

c     CALL SLEEP(2)

c     NOW READ IN THE DATA!!!
      CALL DATA_READ_OUTER(mean_e_all,std_e_all, five_e_all,
     >             tfive_e_all, med_e_all,sfive_e_all,nfive_e_all)

c     THIS IS WHERE YOU CAN SET YOUR OWN INPUTS...DON'T SCREW UP KP!
c     choose_kp=6 IS EQUIVALENT TO Kp=2-

c     READ IN LIST OF INPUTS FROM FILE
c     choose_rd, choose_kp, choose_en, choose_lt
      model_input_file=
     > '/home/phys/flux_model_outer_v1.0/model_input.txt'

      WRITE(6,*) ''
      WRITE(6,*) '---       READING DATA FROM INPUT FILE        ---'
      WRITE(6,*) model_input_file
      OPEN(UNIT=1, FILE=model_input_file, STATUS='OLD')
      DO 4 I=1,ARR_SIZE
        READ(1,200) (input_arr(J,I),J=1,cols)
 4    CONTINUE
      CLOSE(UNIT=1)
      WRITE(6,*) '---                  DONE                     ---'
      WRITE(6,*) ''

c      WRITE(6,*) 
c     > "      choose_rd        choose_kp     choose_en        choose_lt"
c      DO 5 I=1,ARR_SIZE  
c        WRITE(6,201) (input_arr(J,I),J=1,cols)
c 5    CONTINUE

c     NOW OPEN A FILE TO SEND THE OUTPUT TO
      model_output_file=
     > '/home/phys/flux_model_outer_v1.0/model_output.txt'

      OPEN(UNIT=12, FILE=model_output_file, STATUS='UNKNOWN')
c     WRITE TO FILE
      WRITE(12,27), '-------------------------------------------'
c      WRITE(12,*) head_arr2(1)
      WRITE(12,*) head_arr2(1)
      WRITE(12,*) head_arr1(1), str_arr
      WRITE(12,27), '------------------------------------------'

      DO 6 I=1,ARR_SIZE
        choose_rd=input_arr(1,I)
        choose_kp=input_arr(2,I)
        choose_en=input_arr(3,I)
        choose_lt=input_arr(4,I)

c       SEND KP TO CALC_OUTER AS AN INTEGER
        this_kp=INT(choose_kp)

        IF(this_kp.GT.5) this_kp=5
c       ADJUST KP to fit FORTRAN starting index of 1!
        this_kp=this_kp+1

c       ONLY ALLOW VALID INPUTS
        IF(choose_rd.GT.6.D0.AND.choose_rd.LT.20.AND.
     >     choose_lt.GT.0.AND.choose_lt.LT.24.D0)
     >  THEN

c       PERFORM THE INTERPOLATION AND RETURN THE VALUES
          CALL CALC_OUTER(i, choose_rd, this_kp, choose_en, choose_lt,
     >                 mean_e_all,std_e_all, five_e_all,
     >                 tfive_e_all, med_e_all,sfive_e_all,nfive_e_all,
     >                 final_e_flux_all)

c          WRITE(6,*) ''
c          WRITE(6,*) '===================================================='
c          WRITE(6,30)'    CHOSEN KP     =',str_kp(choose_kp),'(INDEX=', 
c         > kp_array(choose_kp),')'
c          WRITE(6,40)'    CHOSEN ENERGY =',choose_en,  ' eV'
c          WRITE(6,50)'    CHOSEN LT     =',choose_lt,  ' LT'
c          WRITE(6,50)'    CHOSEN RADIUS =',choose_rd,  ' R_E'
c          WRITE(6,*) '===================================================='
c          WRITE(6,*) ''

c    PRINT OUT ELECTRON VALUES TO SCREEN
c          WRITE(6,*) '*** ELECTRONS ***'
c          WRITE(6,20) head_arr(1)
c          WRITE(6,10) str_arr(1), final_e_flux_all(1)
c          WRITE(6,10) str_arr(2), final_e_flux_all(2)
c          WRITE(6,10) str_arr(3), final_e_flux_all(3)
c          WRITE(6,10) str_arr(4), final_e_flux_all(4)
c          WRITE(6,10) str_arr(5), final_e_flux_all(5)
c          WRITE(6,10) str_arr(6), final_e_flux_all(6)
c          WRITE(6,10) str_arr(7), final_e_flux_all(7)
c          WRITE(6,*) ''

c     PRINT OUT ELECTRON VALUES TO FILE

          WRITE(12,11) choose_rd, choose_kp, choose_en, choose_lt,
     >                  final_e_flux_all(1), final_e_flux_all(2), 
     >                  final_e_flux_all(3), final_e_flux_all(4), 
     >                  final_e_flux_all(5), final_e_flux_all(6), 
     >                  final_e_flux_all(7)

        ENDIF

 6    CONTINUE

      WRITE(6,*) '          RESULTS WRITTEN TO OUTPUT FILE           '
      WRITE(6,*) model_output_file

      CLOSE(UNIT=12)

c     PROGRAM COMPLETE
      WRITE(6,*) ''
      WRITE(6,*) '-------------FLUX MODEL OUTER COMPLETE-------------'
      WRITE(6,*) ''

 10   FORMAT(A22, F18.6)
 11   FORMAT(4F12.4, 7F18.6)

 20   FORMAT(17x,  4A18)
 25   FORMAT(A18,  F18.6)
 27   FORMAT(A40)
 29   FORMAT(3F18.6)

 30   FORMAT(A20,A3,2x,A7,I2,A1)
 40   FORMAT(A20,F10.3,A4)
 50   FORMAT(A20,F6.2,A5)
 200  FORMAT(4D16.6)
 201  FORMAT(4F16.6)

      STOP
 
      END
c=======================END FLUX MODEL OUTER===================================


